﻿using System.Data;
using System.Reflection;
using Xunit.Sdk;

[AttributeUsage(AttributeTargets.Method, AllowMultiple = false, Inherited = false)]
public sealed class ExcelDataAttribute : DataAttribute
{
    public ExcelDataAttribute(string fileName, string sheetName)
    {
        FileName = fileName;
        SheetName = sheetName;
    }

    public string FileName { get; private set; }

    public string SheetName { get; private set; }

    public override IEnumerable<object[]> GetData(MethodInfo methodInfo)
    {
        if (methodInfo == null)
        {
            throw new ArgumentNullException(nameof(methodInfo));
        }

        ParameterInfo[] parameters = methodInfo.GetParameters();
        return DataSource(FileName, SheetName, parameters.Select(p => p.ParameterType).ToArray());
    }

    private IEnumerable<object[]> DataSource(string fileName, string sheetName, Type[] parameterTypes)
    {
        var path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, fileName);
        var dataSet = ExcelHelper.GetDataSet(fileName);
        if (!string.IsNullOrEmpty(sheetName) && dataSet.Tables.Contains(sheetName))
        {
            var rows = dataSet.Tables[sheetName].Rows;

            foreach (DataRow row in rows)
            {
                yield return ConvertParameters(row.ItemArray, parameterTypes);
            }
        }
    }


    private static object[] ConvertParameters(object[] values, Type[] parameterTypes)
    {
        if (parameterTypes.Length == 0)
            throw new ArgumentNullException(nameof(parameterTypes));

        object[] result = new object[values.Length];

        for (int i = 0; i < values.Length; i++)
            result[i] = Convert.ChangeType(values[i], parameterTypes[i]);

        return result;
    }
}
