﻿
using System.Data;
using OfficeOpenXml;

public class ExcelHelper
{
    public static DataSet GetDataSet(string path)
    {
        if (!File.Exists(path)) throw new FileNotFoundException(path);

        DataSet dataSet = new DataSet();
        using (ExcelPackage package = new ExcelPackage(new FileInfo(path)))
        {
            foreach (var sheet in package.Workbook.Worksheets)
            {
                DataTable table = new DataTable();
                table.TableName = sheet.Name;
                for (int col = 1, k = sheet.Dimension.End.Column; col <= k; col++)
                {
                    var cellCol = sheet.Cells[1, col];
                    if (!string.IsNullOrEmpty(cellCol?.Text))
                        table.Columns.Add(new DataColumn(cellCol.Text));
                }

                for (int row = 2, n = sheet.Dimension.End.Row; row <= n; row++)
                {
                    var dataRow = table.NewRow();
                    for (int col = 1, k = sheet.Dimension.End.Column; col <= k; col++)
                    {
                        var cellVal = sheet.Cells[row, col];
                        var cellCol = sheet.Cells[1, col];
                        if (!string.IsNullOrEmpty(cellCol?.Text))
                        {
                            dataRow[cellCol.Text] = cellVal?.Value;
                        }
                    }
                    table.Rows.Add(dataRow);
                }
                dataSet.Tables.Add(table);
            }
        }
        return dataSet;
    }
    public static List<Dictionary<string, object>> GetData(string path, string sheetName)
    {
        var table = new List<Dictionary<string, object>>();
        if (!File.Exists(path)) throw new FileNotFoundException(path);

        using (ExcelPackage package = new ExcelPackage(new FileInfo(path)))
        {
            var sheet = package.Workbook.Worksheets.FirstOrDefault(w => w.Name.Equals(sheetName, StringComparison.OrdinalIgnoreCase));
            if (sheet == null) throw new ArgumentException($"{nameof(sheetName)}无效");


            for (int row = 2, n = sheet.Dimension.End.Row; row <= n; row++)
            {
                var dataRow = new Dictionary<string, object>();
                for (int col = 1, k = sheet.Dimension.End.Column; col <= k; col++)
                {
                    var cellVal = sheet.Cells[row, col];
                    var cellCol = sheet.Cells[1, col];
                    if (!string.IsNullOrEmpty(cellCol?.Text) && cellVal?.Value != null)
                    {
                        dataRow[cellCol.Text] = cellVal?.Value;
                    }

                }
                table.Add(dataRow);
            }
        }
        return table;
    }
}